{% extends "admin_tools/dashboard/module.html" %}
{% block module_content %}
<ul>
    {% spaceless %}
    {% for child in module.children %}
    <li class="{% cycle 'odd' 'even' %}">
        <span class="float-right">{{ child.action_time|date }}</span>
        {% if child.is_deletion %}
        <span class="deletelink">{% if child.content_type %}{% filter capfirst %}{{ child.content_type.name }}{% endfilter %}&nbsp;{% endif %}{{ child.object_repr }}</span>
        {% else %}
        <a href="{{ child.get_admin_url }}" class="{% if child.is_addition %} addlink{% endif %}{% if child.is_change %} changelink{% endif %}">{% if child.content_type %}{% filter capfirst %}{{ child.content_type.name }}{% endfilter %}&nbsp;{% endif %}{{ child.object_repr }}</a>
        {% endif %}
    </li>
    {% endfor %}
    {% endspaceless %}
</ul>
{% endblock %}
